@@ -1,5 +1,6 @@
 /* keylist.c
- * Copyright (C) 1998,1999,2000,2001,2002,2003 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+ *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -488,8 +489,8 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
                 printf("uid%*s", 28, "");
 
             if ( node->pkt->pkt.user_id->is_revoked )
-                fputs ("[revoked] ", stdout);
-            if ( node->pkt->pkt.user_id->is_expired )
+                fputs (_("[revoked] "), stdout);
+            else if ( node->pkt->pkt.user_id->is_expired )
                 fputs ("[expired] ", stdout);
             print_utf8_string( stdout,  node->pkt->pkt.user_id->name,
                                node->pkt->pkt.user_id->len );
@@ -522,9 +523,15 @@ list_keyblock_print ( KBNODE keyblock, int secret, int fpr, void *opaque )
                    pubkey_letter( pk2->pubkey_algo ),
                    (ulong)keyid2[1],
                    datestr_from_pk( pk2 ) );
-            if( pk2->expiredate ) {
+	    /* Yes, this is an odd way to print the revoked string,
+	       but we already have translations for "[revoked] " (with
+	       the trailing space) and this is a simple way to take
+	       advantage of it.  In devel, this will be done rather
+	       more elegantly. */
+	    if( pk2->is_revoked )
+	        printf(" %s",_("[revoked] "));
+            else if( pk2->expiredate )
                 printf(_(" [expires: %s]"), expirestr_from_pk( pk2 ) );
-            }
             putchar('\n');
 	    if( fpr > 1 )
 		print_fingerprint( pk2, NULL, 0 );
@@ -696,15 +703,12 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
                 ulti_hack = 1;
             putchar(trustletter);
         }
-        printf(":%u:%d:%08lX%08lX:%s:%s:",
+        printf(":%u:%d:%08lX%08lX:%s:%s::",
 		    nbits_from_pk( pk ),
 		    pk->pubkey_algo,
 		    (ulong)keyid[0],(ulong)keyid[1],
 		    colon_datestr_from_pk( pk ),
 		    colon_strtime (pk->expiredate) );
-        if( pk->local_id )
-            printf("%lu", pk->local_id );
-        putchar(':');
         if( !opt.fast_list_mode && !opt.no_expensive_trust_checks  )
             putchar( get_ownertrust_info(pk) );
 	    putchar(':');
@@ -805,7 +809,7 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 	        if(trustletter)
 		  printf("%c", trustletter );
             }
-            printf(":%u:%d:%08lX%08lX:%s:%s:",
+            printf(":%u:%d:%08lX%08lX:%s:%s:::::",
 			nbits_from_pk( pk2 ),
 			pk2->pubkey_algo,
 			(ulong)keyid2[0],(ulong)keyid2[1],
@@ -813,12 +817,6 @@ list_keyblock_colon( KBNODE keyblock, int secret, int fpr )
 			colon_strtime (pk2->expiredate)
 			/* fixme: add LID and ownertrust here */
 						);
-            if( pk->local_id ) /* use the local_id of the main key??? */
-                printf("%lu", pk->local_id );
-            putchar(':');
-            putchar(':');
-            putchar(':');
-            putchar(':');
             print_capabilities (pk2, NULL, NULL);
             putchar('\n');
 	    if( fpr > 1 )
@@ -1136,10 +1134,11 @@ void set_attrib_fd(int fd)
   else if( fd == 2 )
     attrib_fp = stderr;
   else
-    attrib_fp = fdopen( fd, "w" );
+    attrib_fp = fdopen( fd, "wb" );
   if( !attrib_fp ) {
     log_fatal("can't open fd %d for attribute output: %s\n",
 	      fd, strerror(errno));
   }
+
   last_fd = fd;
 }
